@@ -279,6 +279,14 @@ class GitConfig(QtCore.QObject):
     def get_user_or_system(self, key, default=None):
         return self._get(self._global_or_system, key, default)
 
+    def get_object_format(self):
+        """Return the cached repostiory object format (sha256, sha1)"""
+        try:
+            object_format = self._get_value(self._all, 'extensions.objectformat')
+        except KeyError:
+            object_format = 'sha1'
+        return object_format
+
     def set_user(self, key, value):
         if value in (None, ''):
             self.git.config('--global', key, unset=True, _readonly=True)
