@@ -1,17 +1,23 @@
 """Miscellaneous utility functions"""
 import copy
+import hashlib
 import os
 import re
 import shlex
+import shutil
 import sys
 import tempfile
 import time
 import traceback
+import urllib.parse
 
 from . import core
 from . import compat
 
 
+_SSH_REGEX = re.compile(r'^(?P<user>[^@]+)@(?P<hostname>[^:]+):(?P<path>.+)')
+
+
 def asint(obj, default=0):
     """Make any value into an int, even if the cast fails"""
     try:
@@ -26,6 +32,11 @@ def clamp(value, low, high):
     return min(high, max(low, value))
 
 
+def clamp_zero(value, maximum):
+    """Clamp a value between 0 and the maximum value"""
+    return clamp(value, 0, maximum)
+
+
 def epoch_millis():
     return int(time.time() * 1000)
 
@@ -283,6 +294,35 @@ def tmp_filename(label, suffix=''):
         return handle.name
 
 
+def find_bash_exe():
+    """
+    Locate bash.exe bundled with Git for Windows.
+
+    Git for Windows ships its own bash.exe, but it is not added to PATH.
+    This function finds git.exe via PATH, derives the Git installation root,
+    and searches typical subdirectories for bash.exe.
+    """
+    # Find git.exe in PATH
+    git_path = shutil.which('git.exe') or shutil.which('git')
+    if not git_path:
+        return None
+
+    # Estimate Git root directory (e.g., C:\Program Files\Git)
+    cmd_dir = os.path.dirname(os.path.abspath(git_path))
+    git_root = os.path.dirname(cmd_dir)
+
+    # Typical relative locations for bash.exe under Git for Windows
+    candidates = ['bin', 'mingw64/bin', 'usr/bin']
+
+    # Search and return the first match
+    for rel in candidates:
+        bash = os.path.join(git_root, rel, 'bash.exe')
+        if os.path.exists(bash):
+            return bash.replace('\\', '/')
+
+    return None
+
+
 def is_linux():
     """Is this a Linux machine?"""
     return sys.platform.startswith('linux')
@@ -327,6 +367,79 @@ def expandpath(path):
     return path
 
 
+def get_hostname_from_url(url_str):
+    """Extract the hostname component from a URL
+
+    >>> get_hostname_from_url('user@example.org:namespace/project')
+    'example.org'
+
+    >>> get_hostname_from_url('user@example.org:namespace/project.git/')
+    'example.org'
+
+    >>> get_hostname_from_url('https://example.org/namespace/project.git')
+    'example.org'
+
+    >>> get_hostname_from_url('ssh://user@example.org/namespace/project.git')
+    'example.org'
+
+    """
+    if not url_str:
+        return None
+    url = urllib.parse.urlparse(url_str)
+    if not url or url.scheme == '' or url.netloc == '':
+        # Handle <user>@<host>:path.git URLs
+        match = _SSH_REGEX.match(url_str)
+        if match is None:
+            return None
+        hostname = match.group('hostname')
+        if not hostname:
+            return None
+    else:
+        hostname = url.netloc
+
+    if '@' in hostname:
+        _, hostname = hostname.split('@', 1)
+
+    return hostname
+
+
+def get_path_from_url(url_str):
+    """Extract the path component from a URL
+
+    >>> get_path_from_url('user@example.org:namespace/project')
+    'namespace/project'
+
+    >>> get_path_from_url('user@example.org:namespace/project.git/')
+    'namespace/project'
+
+    >>> get_path_from_url('https://example.org/namespace/project.git')
+    'namespace/project'
+
+    >>> get_path_from_url('ssh://user@example.org/namespace/project.git')
+    'namespace/project'
+
+    """
+    if not url_str:
+        return None
+    url = urllib.parse.urlparse(url_str)
+    if not url or url.scheme == '' or url.path == '':
+        # Handle <user>@<host>:path.git URLs
+        match = _SSH_REGEX.match(url_str)
+        if match is None:
+            return None
+        path = match.group('path')
+        if not path:
+            return None
+    else:
+        path = url.path
+
+    path = path.strip('/')  # Strip leading and trailing slashes.
+    if path.endswith('.git'):
+        path = path[: -len('.git')]
+
+    return path
+
+
 class Group:
     """Operate on a collection of objects as a single unit"""
 
@@ -471,3 +584,10 @@ def catch_runtime_error(func, *args, **kwargs):
         valid = False
         result = None
     return (valid, result)
+
+
+def sha256hex(value):
+    """Return a hexdigest of the SHA-256 hash of the specified value"""
+    hash = hashlib.new('sha256')
+    hash.update(value.encode('utf-8', 'replace'))
+    return hash.hexdigest()
