@@ -51,10 +51,10 @@ def git_dag(context, args=None, existing_view=None, show=True):
     else:
         view = existing_view
         view.set_params(params)
-    if params.ref:
-        view.display()
     if show:
         view.show()
+    if params.ref:
+        view.display()
     return view
 
 
@@ -734,7 +734,9 @@ class CommitTreeWidget(standard.TreeWidget, ViewerMixin):
             self, N_('Zoom to Fit'), self.zoom_to_fit.emit, hotkeys.FIT
         )
 
-        self.itemSelectionChanged.connect(self.selection_changed)
+        self.itemSelectionChanged.connect(
+            self.selection_changed, type=Qt.QueuedConnection
+        )
 
     def export_state(self):
         """Export the widget's state"""
@@ -898,6 +900,7 @@ class GitDAG(standard.MainWindow):
         self.commits = {}
         self.commit_list = []
         self.selection = []
+        self.old_selection = []
         self.old_refs = set()
         self.old_oids = None
         self.old_count = 0
@@ -907,7 +910,7 @@ class GitDAG(standard.MainWindow):
 
         self.thread = None
         self.revtext = GitDagLineEdit(context)
-        self.maxresults = standard.SpinBox()
+        self.maxresults = standard.SpinBox(digits=None, maxi=9999999, wrap=True)
 
         self.zoom_out = qtutils.create_action_button(
             tooltip=N_('Zoom Out'), icon=icons.zoom_out()
@@ -926,18 +929,35 @@ class GitDAG(standard.MainWindow):
         self.filewidget = filelist.FileWidget(context, self)
         self.graphview = GraphView(context, self)
 
-        self.treewidget.commits_selected.connect(self.commits_selected)
-        self.graphview.commits_selected.connect(self.commits_selected)
+        self.treewidget.commits_selected.connect(
+            self.commits_selected, type=Qt.QueuedConnection
+        )
+        self.graphview.commits_selected.connect(
+            self.commits_selected, type=Qt.QueuedConnection
+        )
 
-        self.commits_selected.connect(self.select_commits)
-        self.commits_selected.connect(self.diffwidget.commits_selected)
-        self.commits_selected.connect(self.filewidget.commits_selected)
-        self.commits_selected.connect(self.graphview.select_commits)
-        self.commits_selected.connect(self.treewidget.select_commits)
-
-        self.filewidget.files_selected.connect(self.diffwidget.files_selected)
-        self.filewidget.difftool_selected.connect(self.difftool_selected)
-        self.filewidget.histories_selected.connect(self.histories_selected)
+        self.commits_selected.connect(self.select_commits, type=Qt.QueuedConnection)
+        self.commits_selected.connect(
+            self.diffwidget.commits_selected, type=Qt.QueuedConnection
+        )
+        self.commits_selected.connect(
+            self.filewidget.commits_selected, type=Qt.QueuedConnection
+        )
+        self.commits_selected.connect(
+            self.graphview.select_commits, type=Qt.QueuedConnection
+        )
+        self.commits_selected.connect(
+            self.treewidget.select_commits, type=Qt.QueuedConnection
+        )
+        self.filewidget.files_selected.connect(
+            self.diffwidget.files_selected, type=Qt.QueuedConnection
+        )
+        self.filewidget.difftool_selected.connect(
+            self.difftool_selected, type=Qt.QueuedConnection
+        )
+        self.filewidget.histories_selected.connect(
+            self.histories_selected, type=Qt.QueuedConnection
+        )
 
         self.proxy = FocusRedirectProxy(
             self.treewidget, self.graphview, self.filewidget
@@ -1048,21 +1068,31 @@ class GitDAG(standard.MainWindow):
         qtutils.connect_button(self.zoom_in, self.graphview.zoom_in)
         qtutils.connect_button(self.zoom_to_fit, self.graphview.zoom_to_fit)
 
-        self.treewidget.zoom_to_fit.connect(self.graphview.zoom_to_fit)
-        self.treewidget.diff_commits.connect(self.diff_commits)
-        self.treewidget.search_line_range_in_oid.connect(self.search_line_range_in_oid)
-        self.graphview.diff_commits.connect(self.diff_commits)
-        self.graphview.search_line_range_in_oid.connect(self.search_line_range_in_oid)
-        self.filewidget.grab_file.connect(self.grab_file)
-        self.filewidget.grab_file_from_parent.connect(self.grab_file_from_parent)
-        self.filewidget.select_line_range_for_file.connect(
-            self.search_line_range_for_file
+        self.treewidget.zoom_to_fit.connect(
+            self.graphview.zoom_to_fit, type=Qt.QueuedConnection
         )
-        self.maxresults.editingFinished.connect(self.display)
-        self.revtext.textChanged.connect(self.text_changed)
-        self.revtext.activated.connect(self.display)
-        self.revtext.enter.connect(self.display)
-        self.revtext.down.connect(self.focus_tree)
+        self.treewidget.diff_commits.connect(
+            self.diff_commits, type=Qt.QueuedConnection
+        )
+        self.treewidget.search_line_range_in_oid.connect(
+            self.search_line_range_in_oid, type=Qt.QueuedConnection
+        )
+        self.graphview.diff_commits.connect(self.diff_commits, type=Qt.QueuedConnection)
+        self.graphview.search_line_range_in_oid.connect(
+            self.search_line_range_in_oid, type=Qt.QueuedConnection
+        )
+        self.filewidget.grab_file.connect(self.grab_file, type=Qt.QueuedConnection)
+        self.filewidget.grab_file_from_parent.connect(
+            self.grab_file_from_parent, type=Qt.QueuedConnection
+        )
+        self.filewidget.select_line_range_for_file.connect(
+            self.search_line_range_for_file, type=Qt.QueuedConnection
+        )
+        self.maxresults.editingFinished.connect(self.display, type=Qt.QueuedConnection)
+        self.revtext.textChanged.connect(self.text_changed, type=Qt.QueuedConnection)
+        self.revtext.activated.connect(self.display, type=Qt.QueuedConnection)
+        self.revtext.enter.connect(self.display, type=Qt.QueuedConnection)
+        self.revtext.down.connect(self.focus_tree, type=Qt.QueuedConnection)
         # The model is updated in another thread so use
         # signals/slots to bring control back to the main GUI thread
         self.model.updated.connect(self.model_updated, type=Qt.QueuedConnection)
@@ -1082,15 +1112,22 @@ class GitDAG(standard.MainWindow):
         self.maxresults.setValue(params.count)
         self.update_window_title()
 
-        if self.thread is not None:
-            self.thread.stop()
-        self.thread = thread = ReaderThread(context, params, self)
-        thread.begin.connect(self.thread_begin, type=Qt.QueuedConnection)
-        thread.status.connect(self.thread_status, type=Qt.QueuedConnection)
-        thread.add.connect(self.add_commits, type=Qt.QueuedConnection)
-        thread.end.connect(self.thread_end, type=Qt.QueuedConnection)
+        self._stop_reader_thread()
+        self.thread = ReaderThread(context, params)
+        self.thread.begin.connect(self.thread_begin, type=Qt.QueuedConnection)
+        self.thread.status.connect(self.thread_status, type=Qt.QueuedConnection)
+        self.thread.add.connect(self.add_commits, type=Qt.QueuedConnection)
+        self.thread.end.connect(self.thread_end, type=Qt.QueuedConnection)
+
+    def _stop_reader_thread(self):
+        """Stop the reader thread if it is currently running"""
+        if self.thread is not None and self.thread.isRunning():
+            self.thread.requestInterruption()
+            QtCore.QThread.currentThread().yieldCurrentThread()
+            self.thread.wait(100)
 
     def _enable_worktree_status(self, enabled):
+        """Enable and disable the display of the WORKTREE and STAGE pseudo-commits"""
         self.params.display_status = enabled
         self.display()
 
@@ -1107,10 +1144,12 @@ class GitDAG(standard.MainWindow):
         self.diffwidget.setFocus()
 
     def text_changed(self, txt):
+        """Respond to changes to the revision input text"""
         self.params.ref = txt
         self.update_window_title()
 
     def update_window_title(self):
+        """Update the window title to reflect the displayed ref"""
         project = self.model.project
         if self.params.ref:
             self.setWindowTitle(
@@ -1124,6 +1163,7 @@ class GitDAG(standard.MainWindow):
             self.setWindowTitle(project + N_(' - DAG'))
 
     def export_state(self):
+        """Store persistent GUI state"""
         state = standard.MainWindow.export_state(self)
         state['count'] = self.params.count
         state['display_status'] = self.params.display_status
@@ -1132,6 +1172,7 @@ class GitDAG(standard.MainWindow):
         return state
 
     def apply_state(self, state):
+        """Apply persistent GUI state"""
         result = standard.MainWindow.apply_state(self, state)
         try:
             count = state['count']
@@ -1200,7 +1241,7 @@ class GitDAG(standard.MainWindow):
             or display_status != self.old_display_status
         )
         if update:
-            self.thread.stop()
+            self._stop_reader_thread()
             self.params.set_ref(ref)
             self.params.set_count(count)
             self.params.set_display_status(display_status)
@@ -1213,37 +1254,50 @@ class GitDAG(standard.MainWindow):
         self.force_refresh = False
 
     def select_commits(self, commits):
+        """Commits were selected"""
         self.selection = commits
         enabled = bool(commits)
         self.diffwidget_copy_commit.setEnabled(enabled)
 
     def clear(self):
+        """Clear the view and the list of known commits"""
         self.commits.clear()
         self.commit_list = []
         self.graphview.clear()
         self.treewidget.clear()
 
     def add_commits(self, commits):
+        """Add new commits from the reader thread"""
         self.commit_list.extend(commits)
         # Keep track of commits
         for commit_obj in commits:
             self.commits[commit_obj.oid] = commit_obj
             for tag in commit_obj.tags:
                 self.commits[tag] = commit_obj
-        self.graphview.add_commits(commits)
+        # The treewidget is quick to update.  The graphview is slower when updating
+        # incrementally so it is updated just once at thread_end() once all commits have
+        # been gathered.
         self.treewidget.add_commits(commits)
 
     def thread_begin(self):
+        """The reader thread has begun"""
+        if self.selection:
+            self.old_selection = self.selection
         self.clear()
 
     def thread_end(self):
+        """The reader thread has completed"""
+        self.graphview.add_commits(self.commit_list)
         self.restore_selection()
 
     def thread_status(self, successful):
+        """Indicate an error when the revision input contains an invalid ref"""
         self.revtext.hint.set_error(not successful)
 
     def restore_selection(self):
-        selection = self.selection
+        """Restore the selection before the display was refreshed"""
+        # The selection can become empty when the widgets are cleared.
+        selection = self.selection or self.old_selection
         try:
             commit_obj = self.commit_list[-1]
         except IndexError:
@@ -1297,6 +1351,7 @@ class GitDAG(standard.MainWindow):
         self.display()
 
     def histories_selected(self, histories):
+        """Respond to file-based history selection from the files widget"""
         argv = [self.model.currentbranch, '--']
         argv.extend(histories)
         rev_text = core.list2cmdline(argv)
@@ -1304,6 +1359,7 @@ class GitDAG(standard.MainWindow):
         self.display()
 
     def difftool_selected(self, files):
+        """Launch difftool across a commit range"""
         bottom, top = self.treewidget.selected_commit_range()
         if not top:
             return
@@ -1332,9 +1388,9 @@ class GitDAG(standard.MainWindow):
 
     # Qt overrides
     def closeEvent(self, event):
-        """Ensure the revtext popup is closed"""
+        """Ensure the revision text popup is closed"""
         self.revtext.close_popup()
-        self.thread.stop()
+        self._stop_reader_thread()
         standard.MainWindow.closeEvent(self, event)
 
     def showEvent(self, event):
@@ -1351,32 +1407,25 @@ class ReaderThread(QtCore.QThread):
     end = Signal()
     status = Signal(object)
 
-    def __init__(self, context, params, parent):
-        QtCore.QThread.__init__(self, parent)
+    def __init__(self, context, params):
+        super().__init__()
         self.context = context
         self.params = params
-        self._stop = False
-        self._mutex = QtCore.QMutex()
-        self._condition = QtCore.QWaitCondition()
 
     def run(self):
+        """Gather commits and emit them to the main thread"""
         context = self.context
         repo = dag.RepoReader(context, self.params)
         repo.reset()
         self.begin.emit()
+
         commits = []
         for commit in repo.get():
-            stopped = False
-            self._mutex.lock()
-            if self._stop:
-                self._condition.wait(self._mutex)
+            if self.isInterruptionRequested():
                 repo.reset()
-                stopped = True
-            self._mutex.unlock()
-            if stopped:
                 return
             commits.append(commit)
-            if len(commits) >= 512:
+            if len(commits) >= 2048:
                 self.add.emit(commits)
                 commits = []
 
@@ -1391,14 +1440,6 @@ class ReaderThread(QtCore.QThread):
         self.status.emit(repo.returncode == 0)
         self.end.emit()
 
-    def start(self):
-        self._stop = False
-        QtCore.QThread.start(self)
-
-    def stop(self):
-        self._stop = True
-        self.wait()
-
 
 class Cache:
     _label_font = None
@@ -1848,7 +1889,7 @@ class GraphView(QtWidgets.QGraphicsView, ViewerMixin):
 
         scene = QtWidgets.QGraphicsScene(self)
         scene.setItemIndexMethod(QtWidgets.QGraphicsScene.BspTreeIndex)
-        scene.selectionChanged.connect(self.selection_changed)
+        scene.selectionChanged.connect(self.selection_changed, type=Qt.QueuedConnection)
         self.setScene(scene)
 
         self.setRenderHint(QtGui.QPainter.Antialiasing)
@@ -1894,6 +1935,7 @@ class GraphView(QtWidgets.QGraphicsView, ViewerMixin):
     def clear(self):
         EdgeColor.reset()
         self.scene().clear()
+        self.scene().invalidate()
         self.items.clear()
         self.x_offsets.clear()
         self.x_min = 24
@@ -2326,7 +2368,7 @@ class GraphView(QtWidgets.QGraphicsView, ViewerMixin):
             # Align new column frontier by frontier of nearest column. If all
             # columns were left then select maximum frontier value.
             if not self.columns:
-                self.frontier[column] = max(list(self.frontier.values()))
+                self.frontier[column] = max(self.frontier.values())
                 return
             # This is heuristic that mostly affects roots. Note that the
             # frontier values for fork children will be overridden in course of
@@ -2406,9 +2448,9 @@ class GraphView(QtWidgets.QGraphicsView, ViewerMixin):
 
         # Avoid overlapping with tags of commits at cell_row.
         if self.x_off > 0:
-            can_overlap = list(range(self.min_column, column))
+            can_overlap = range(self.min_column, column)
         else:
-            can_overlap = list(range(self.max_column, column, -1))
+            can_overlap = range(self.max_column, column, -1)
         for cell_row in itertools.count(cell_row):
             for value in can_overlap:
                 if (value, cell_row) in self.tagged_cells:
