@@ -7,6 +7,7 @@ from .. import cmds
 from .. import hidpi
 from .. import icons
 from .. import qtutils
+from .. import spellcheck
 from .. import themes
 from ..compat import ustr
 from ..i18n import N_
@@ -174,6 +175,13 @@ scissors
         self.inotify = qtutils.checkbox(checked=True)
         self.inotify.setToolTip(tooltip)
 
+        tooltip = N_(
+            'Milliseconds to wait between filesystem change events.\n'
+            'Hint: 5000 ms is 5 seconds. 60000 ms is one minute. 360000 ms is 1 hour.'
+        )
+        # 360000 milliseconds is 1 hour (60mins * 60secs * 1000ms)
+        self.inotify_delay = standard.SpinBox(mini=100, maxi=3600000, tooltip=tooltip)
+
         self.logdate = qtutils.combo(prefs.date_formats())
         tooltip = N_(
             'The date-time format used when displaying dates in Git DAG.\n'
@@ -190,37 +198,45 @@ scissors
         tooltip = N_('Enable path autocompletion in tools')
         self.autocomplete_paths = qtutils.checkbox(checked=True, tooltip=tooltip)
 
-        self.proxy_spacer = QtWidgets.QLabel()
         tooltip = N_('Enable detection and configuration of HTTP proxy settings')
         self.autodetect_proxy = qtutils.checkbox(checked=True, tooltip=tooltip)
 
         self.add_row(N_('Name'), self.name)
         self.add_row(N_('Email'), self.email)
+
+        self.add_row('', QtWidgets.QLabel())
         self.add_row(N_('Patches Directory'), self.patches_directory)
-        self.add_row(N_('Log Date Format'), self.logdate)
-        self.add_row(N_('Commit Message Cleanup'), self.commit_cleanup)
-        self.add_row(N_('Merge Verbosity'), self.merge_verbosity)
         self.add_row(N_('Number of Diff Context Lines'), self.diff_context)
-        self.add_row(N_('Summarize Merge Commits'), self.merge_summary)
-        self.add_row(N_('Enable Filesystem Monitoring'), self.inotify)
+        self.add_row(N_('Log Date Format'), self.logdate)
+
+        self.add_row('', QtWidgets.QLabel())
+        self.add_row(
+            N_('Automatically Load Commit Message Template'), self.autotemplate
+        )
+        self.add_row(N_('Commit Message Cleanup'), self.commit_cleanup)
+
+        self.add_row('', QtWidgets.QLabel())
         self.add_row(
             N_('Update stacked branches/refs when rebasing'),
             self.rebase_update_refs,
         )
-        self.add_row(
-            N_('Automatically Load Commit Message Template'), self.autotemplate
-        )
-        self.add_row(N_('Show Full Paths in the Window Title'), self.show_path)
         self.add_row(N_('Show Diffstat After Merge'), self.merge_diffstat)
+        self.add_row(N_('Summarize Merge Commits'), self.merge_summary)
+        self.add_row(N_('Merge Verbosity'), self.merge_verbosity)
+
+        self.add_row('', QtWidgets.QLabel())
         self.add_row(N_('Display Untracked Files'), self.display_untracked)
+        self.add_row(N_('Enable Filesystem Monitoring'), self.inotify)
+        self.add_row(N_('Filesystem Monitoring Event Delay'), self.inotify_delay)
+        self.add_row(N_('Enable Gravatar Icons'), self.enable_gravatar)
+        self.add_row(N_('Update Index on Startup'), self.update_index)
+        self.add_row(N_('Autocomplete Paths'), self.autocomplete_paths)
+        self.add_row(N_('Show Full Paths in the Window Title'), self.show_path)
         self.add_row(
             N_('Display desktop notifications using popup dialogs'), self.enable_popups
         )
-        self.add_row(N_('Enable Gravatar Icons'), self.enable_gravatar)
-        self.add_row(N_('Autocomplete Paths'), self.autocomplete_paths)
-        self.add_row(N_('Update Index on Startup'), self.update_index)
 
-        self.add_row('', self.proxy_spacer)
+        self.add_row('', QtWidgets.QLabel())
         self.add_row(
             N_('Automatically Detect and Configure Proxy Settings'),
             self.autodetect_proxy,
@@ -247,6 +263,7 @@ scissors
             prefs.ENABLE_POPUPS: (self.enable_popups, Defaults.enable_popups),
             prefs.HTTP_PROXY: (self.http_proxy, Defaults.http_proxy),
             prefs.INOTIFY: (self.inotify, Defaults.inotify),
+            prefs.INOTIFY_DELAY: (self.inotify_delay, Defaults.inotify_delay),
             prefs.LOGDATE: (self.logdate, Defaults.logdate),
             prefs.MERGE_DIFFSTAT: (self.merge_diffstat, Defaults.merge_diffstat),
             prefs.MERGE_SUMMARY: (self.merge_summary, Defaults.merge_summary),
@@ -275,8 +292,7 @@ class SettingsFormWidget(FormWidget):
 
         self.editor = QtWidgets.QLineEdit()
         self.editor.setToolTip(N_('The main GUI editor that must block until it exits'))
-        tooltip = N_('Use "aspell" as the spelling dictionary source')
-        self.aspell_enabled = qtutils.checkbox(tooltip=tooltip)
+
         self.background_editor = QtWidgets.QLineEdit()
         self.background_editor.setToolTip(
             N_('A non-blocking GUI editor that is launched in the background')
@@ -291,7 +307,6 @@ class SettingsFormWidget(FormWidget):
         self.keep_merge_backups = qtutils.checkbox()
         self.sort_bookmarks = qtutils.checkbox()
         self.save_window_settings = qtutils.checkbox()
-        self.check_spelling = qtutils.checkbox()
         tooltip = N_('Detect conflict markers in unmerged files')
         self.check_conflicts = qtutils.checkbox(checked=True, tooltip=tooltip)
         self.expandtab = qtutils.checkbox(tooltip=N_('Insert tabs instead of spaces'))
@@ -307,8 +322,22 @@ class SettingsFormWidget(FormWidget):
         tooltip = N_('Emit notifications when commits are pushed.')
         self.notifyonpush = qtutils.checkbox(checked=False, tooltip=tooltip)
 
+        tooltip = N_('Use "aspell" as the spelling dictionary source')
+        self.aspell_enabled = qtutils.checkbox(tooltip=tooltip)
+        self.check_spelling = qtutils.checkbox()
+        tooltip = N_('Additional spellcheck dictionary files (requires restart)')
+        self.spelling_dictionaries = DictionaryList(context)
+        self.spelling_dictionaries.setToolTip(tooltip)
+
+        tooltip = N_('Set the verbosity level to 1 or higher to log Git commands')
+        self.verbosity = standard.SpinBox(value=0, mini=0, maxi=2)
+
         self.add_row(N_('Text Width'), self.textwidth)
         self.add_row(N_('Tab Width'), self.tabwidth)
+        self.add_row(N_('Insert Spaces Instead of Tabs'), self.expandtab)
+        self.add_row(N_('Auto-Wrap Lines'), self.linebreak)
+
+        self.add_row('', QtWidgets.QLabel())
         self.add_row(N_('Editor'), self.editor)
         self.add_row(N_('Background Editor'), self.background_editor)
         self.add_row(N_('History Browser'), self.historybrowser)
@@ -316,22 +345,30 @@ class SettingsFormWidget(FormWidget):
         self.add_row(N_('Diff Tool'), self.difftool)
         self.add_row(N_('Merge Tool'), self.mergetool)
         self.add_row(N_('Recent Repository Count'), self.maxrecent)
-        self.add_row(N_('Auto-Wrap Lines'), self.linebreak)
-        self.add_row(N_('Insert Spaces Instead of Tabs'), self.expandtab)
+
+        self.add_row('', QtWidgets.QLabel())
+        self.add_row(N_('Detect Conflict Markers'), self.check_conflicts)
+        self.add_row(N_('Keep *.orig Merge Backups'), self.keep_merge_backups)
+
+        self.add_row('', QtWidgets.QLabel())
+        self.add_row(N_('Save GUI Settings'), self.save_window_settings)
+        self.add_row(N_('Ctrl + MouseWheel to Zoom'), self.mouse_zoom)
+        self.add_row(N_('Refresh on Focus'), self.refresh_on_focus)
+        self.add_row(N_('Sort Bookmarks Alphabetically'), self.sort_bookmarks)
+        self.add_row(N_('Resize File Browser columns'), self.resize_browser_columns)
+
+        self.add_row('', QtWidgets.QLabel())
+        self.add_row(N_('Safe Mode'), self.safe_mode)
         self.add_row(
             N_('Check Published Commits when Amending'), self.check_published_commits
         )
-        self.add_row(N_('Sort Bookmarks Alphabetically'), self.sort_bookmarks)
-        self.add_row(N_('Safe Mode'), self.safe_mode)
-        self.add_row(N_('Detect Conflict Markers'), self.check_conflicts)
-        self.add_row(N_('Keep *.orig Merge Backups'), self.keep_merge_backups)
-        self.add_row(N_('Save GUI Settings'), self.save_window_settings)
-        self.add_row(N_('Refresh on Focus'), self.refresh_on_focus)
-        self.add_row(N_('Resize File Browser columns'), self.resize_browser_columns)
+        self.add_row(N_('Notify on Push'), self.notifyonpush)
+        self.add_row(N_('Verbosity Level'), self.verbosity)
+
+        self.add_row('', QtWidgets.QLabel())
         self.add_row(N_('Check Spelling'), self.check_spelling)
         self.add_row(N_('Enable "aspell" Spelling Dictionaries'), self.aspell_enabled)
-        self.add_row(N_('Ctrl + MouseWheel to Zoom'), self.mouse_zoom)
-        self.add_row(N_('Notify on Push'), self.notifyonpush)
+        self.add_row(N_('Additional Spelling Dictionaries'), self.spelling_dictionaries)
 
         self.set_config({
             prefs.ASPELL_ENABLED: (self.aspell_enabled, Defaults.aspell_enabled),
@@ -375,6 +412,7 @@ class SettingsFormWidget(FormWidget):
             prefs.SPELL_CHECK: (self.check_spelling, Defaults.spellcheck),
             prefs.MOUSE_ZOOM: (self.mouse_zoom, Defaults.mouse_zoom),
             prefs.NOTIFY_ON_PUSH: (self.notifyonpush, Defaults.notifyonpush),
+            prefs.VERBOSITY: (self.verbosity, Defaults.verbosity),
         })
 
 
@@ -402,6 +440,7 @@ class AppearanceFormWidget(FormWidget):
         # parsed as a string, so the transform is effectively a no-op.
         self.high_dpi = qtutils.combo_mapped(hidpi.options(), transform=ustr)
         self.high_dpi.setEnabled(hidpi.is_supported())
+        self.bold_fonts = qtutils.checkbox()
         self.bold_headers = qtutils.checkbox()
         self.status_show_totals = qtutils.checkbox()
         self.status_indent = qtutils.checkbox()
@@ -413,12 +452,14 @@ class AppearanceFormWidget(FormWidget):
         self.add_row(N_('GUI theme'), self.theme)
         self.add_row(N_('Icon theme'), self.icon_theme)
         self.add_row(N_('High DPI'), self.high_dpi)
+        self.add_row(N_('Bold all fonts'), self.bold_fonts)
         self.add_row(N_('Bold on Dark Headers Instead of Italic'), self.bold_headers)
         self.add_row(N_('Show File Counts in Status Titles'), self.status_show_totals)
         self.add_row(N_('Indent Status paths'), self.status_indent)
         self.add_row(N_('Use a Block Cursor in Diff Editors'), self.block_cursor)
 
         self.set_config({
+            prefs.BOLD_FONTS: (self.bold_fonts, Defaults.bold_fonts),
             prefs.BOLD_HEADERS: (self.bold_headers, Defaults.bold_headers),
             prefs.FONTSIZE: (
                 self.font_size,
@@ -508,6 +549,110 @@ class AppearanceWidget(QtWidgets.QWidget):
         self.form.update_from_config()
 
 
+class DictionaryList(QtWidgets.QWidget):
+    """A widget for editing a list of spelling dictionaries"""
+
+    def __init__(self, context):
+        QtWidgets.QWidget.__init__(self)
+        self.context = context
+        self.add_button = qtutils.create_toolbutton(
+            text=N_('Add'), icon=icons.add(), tooltip=N_('Add Dictionary')
+        )
+        self.add_menu = qtutils.create_menu(N_('Add Dictionary'), self.add_button)
+        self.add_button.setMenu(self.add_menu)
+
+        self.remove_button = qtutils.create_toolbutton(
+            text=N_('Remove'),
+            icon=icons.remove(),
+            tooltip=N_('Remove selected (Delete)'),
+        )
+        self.remove_button.setEnabled(False)
+
+        self.dict_list = standard.ListWidget()
+
+        self.top_layout = qtutils.hbox(
+            defs.no_margin,
+            defs.spacing,
+            self.add_button,
+            self.remove_button,
+            qtutils.STRETCH,
+        )
+        layout = qtutils.vbox(
+            defs.no_margin, defs.spacing, self.top_layout, self.dict_list
+        )
+        self.setLayout(layout)
+
+        self.add_menu.aboutToShow.connect(self._build_menu)
+        self.dict_list.itemSelectionChanged.connect(self._selection_changed)
+        qtutils.connect_button(self.remove_button, self._remove_selected)
+
+        self.refresh()
+
+    def refresh(self):
+        """Refresh the list of configured dictionaries"""
+        self.dict_list.clear()
+        current_dicts = prefs.spelling_dictionaries(self.context)
+        if current_dicts:
+            self.dict_list.addItems(current_dicts)
+
+    def _selection_changed(self):
+        """Update the widget state when the dictionary selection changes"""
+        items = self.dict_list.selected_items()
+        self.remove_button.setEnabled(bool(items))
+
+    def _build_menu(self):
+        """Populate the "Add" menu with dictionary files"""
+        current_dicts = set(prefs.spelling_dictionaries(self.context))
+        dictionaries = spellcheck.get_available_dictionaries()
+        paths = []
+        for path in dictionaries:
+            if path in current_dicts:
+                continue
+            paths.append(path)
+
+        # If all of the paths were already added, or if the quick list of
+        # dictionaries is empty, then launch the file dialog immediately.
+        self.add_menu.clear()
+        if not paths:
+            self._select_dictionaries()
+            return
+
+        select_action = self.add_menu.addAction(N_('Select dictionary file(s)...'))
+        select_action.triggered.connect(lambda _: self._select_dictionaries())
+        self.add_menu.addSeparator()
+
+        for path in paths:
+            action = self.add_menu.addAction(path)
+            action.triggered.connect(
+                lambda _, path=path: self._add_dictionaries([path])
+            )
+
+    def _remove_selected(self):
+        """Remove selected dictionary files"""
+        values = [item.text() for item in self.dict_list.selected_items()]
+        if not values:
+            return
+        remove_cmd = prefs.RemoveDictionary(self.context, values)
+        remove_cmd.do()
+        self.refresh()
+
+    def _add_dictionaries(self, values):
+        """Add dictionary files"""
+        if not values:
+            return
+        add_command = prefs.AddDictionary(self.context, values)
+        add_command.do()
+        self.refresh()
+
+    def _select_dictionaries(self):
+        """Select and add dictionary files from disk"""
+        values = qtutils.open_files(
+            N_('Select dictionary file(s)...'),
+            filters='Spelling Dictionaries (*.dic);;All Files (*)',
+        )
+        self._add_dictionaries(values)
+
+
 class PreferencesView(standard.Dialog):
     def __init__(self, context, model, parent=None):
         standard.Dialog.__init__(self, parent=parent)
